@@ -18,6 +18,7 @@
 # Contact information: Zmanda Inc, 505 N Mathlida Ave, Suite 120
 # Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
 #
+#
 
 # This is meant to be invoked by xinetd.
 # It expects two arguments on stdin
@@ -59,10 +60,11 @@ my $tmp_directory;
 my $action;
 my $params;
 my $MYSQLHOTCOPY="mysqlhotcopy";
-my $VERSION="1.8";
+my $VERSION="1.9";
 my $logDir = "/var/log/mysql-zrm";
 my $logFile = "$logDir/socket-server.log";
 my $snapshotInstallPath = "/usr/share/mysql-zrm/plugins";
+my $compress = "";
 
 open LOG, ">$logFile" or die "Unable to create log file";
 $SIG{'PIPE'} = sub { &printAndDie( "pipe broke\n" ); };
@@ -70,7 +72,15 @@ $SIG{'PIPE'} = sub { &printAndDie( "pipe broke\n" ); };
 # This will only allow and a-z A-Z 0-9 _ - / . = " ' ; + * and space.
 # Modify this if any other characters are to be allowed.
 sub checkIfTainted(){
-        if( $_[0] =~ /^([-\*\w\/"\'.\:\+\s=\^\$]+)$/) {
+	# The only field where semicolon can exist is in password
+	# So blank out the password while checking for existence of semicolon
+	my $x = $_[0];
+	$x=~s/\s+--password=("[^"]*")\s+/ --password="" /;
+	if( $x =~ /;/ ){
+		&printAndDie("Bad data in $_[0]\n");
+	}
+	
+        if( $_[0] =~ /^([-\*\w\/"\'.\:;\+\s=\^\$]+)$/) {
                 return $1;
         }else{
                 &printAndDie("Bad data in $_[0]\n");
@@ -104,7 +114,7 @@ sub getInputs()
 	if( $x ne $VERSION ){
 		&printAndDie( "Version of remote copy plugin does not match\n" );
 	}
-	for( my $i = 0; $i < 4; $i++ ){
+	for( my $i = 0; $i < 5; $i++ ){
 		$x = <STDIN>;
 		push @inp, $x;	
 	}
@@ -113,6 +123,9 @@ sub getInputs()
 	$params = &checkIfTainted($inp[1]);
 	$TMPDIR = &checkIfTainted($inp[2]);
 	$MYSQL_BINPATH = &checkIfTainted($inp[3]);
+	if( $inp[4] ne "" ){
+		$compress = &checkIfTainted($inp[4]);
+	}
 	if( ! -d $MYSQL_BINPATH || ! -f "$MYSQL_BINPATH/$MYSQLHOTCOPY" ){
 		&printAndDie( "mysqlhotcopy not found\n" );
 	}
@@ -141,7 +154,7 @@ sub writeTarStream()
                 $fileList = " -T $tmpFile";
         }
 
-	unless(open( TAR_H, "$TAR --same-owner -cphszC $_[0] $fileList 2>/dev/null|" ) ){
+	unless(open( TAR_H, "$TAR --same-owner $compress -cphsC $_[0] $fileList 2>/dev/null|" ) ){
 		&printandDie( "tar failed $!\n" );
 	}
 	binmode( TAR_H );
@@ -161,7 +174,7 @@ sub writeTarStream()
 #$_[0] dirname to strea the data to
 sub readTarStream()
 {
-	unless(open( TAR_H, "|$TAR --same-owner -xphszC $_[0] 2>/dev/null" ) ){
+	unless(open( TAR_H, "|$TAR --same-owner $compress -xphsC $_[0] 2>/dev/null" ) ){
 		&printandDie( "tar failed $!\n" );
 	}
 
@@ -266,7 +279,12 @@ sub doCreateLink()
 	for( $i = 0; $i < $num; $i += 2 ){
 		my $p = &readOneLine();
 		my $link = &readOneLine();
-		mkpath( $p, 0, 0700 );
+		eval { mkpath( $p, 0, 0700 ) };
+		if ($@) {
+			&printToServer( "ERROR", "Unable to create directory $p, $@" );
+			&printAndDie( "Unable to create directory $p, $@ \n" );
+		}
+
 		my $cmd = "ln -s $link";
 		my $r = system( $cmd );
 		if( $r != 0 ){
@@ -378,7 +396,10 @@ if( $action eq "copy from" ){
 	}
 	&doCopyBetween();
 }elsif( $action eq "copy to" ){
-	if( ! -d $params ){
+	my $s = $params;
+	$s =~ s/^"//;
+	$s =~ s/"$//;
+	if( ! -d $s ){
 		&printAndDie( "$params not found\n" );
 	}
 	&readTarStream( $params );
